def get_train_tips():
    return [
        {
            "title": "深度学习网络训练技巧汇总",
            "url": "https://zhuanlan.zhihu.com/p/20767428",
        },
        {
            "title": "深度学习调参有哪些技巧？",
            "url": "https://www.zhihu.com/question/25097993",
        },
    ]
